package kd.hrmp.hbjm.business.cascadedata;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.ArrayUtils;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.util.HRStringUtils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * @author rd_ningjie
 * @description:级联对比实体
 * @date 2022/6/6 9:58
 */
public final class CompareEntity {

    public String CON_SYMBOL = "!!";

    //关联字段用来查库或者与其他实体关联的字段
    private String relactionCol;

    //与其他实体进行对比的字段
    private String compareCol;

    //父实体值
    private String value;

    //与父实体进行对比的字段值
    private String compareValue;

    //是否来自于表数据（如果是，则数据源进行查库，如果不是，则使用datalist作为对比数据源）（
    //如果已经有了数据，即使是数据库实体，也可以传false，并对dataList赋值
    private boolean isfromDb = false;

    //实体编码
    private String entityCode;

    //数据源过滤器
    private QFilter[] qFilters;

    //额外需要查询的字段信息
    private String selectExtProps;

    //数据源，如果isfromDb = true，则根据qFilters进行查库，并且采取add的方法进行缓存数据
    private Map<String, DynamicObject> dataContainer;


    /**
     * @param relactionCol
     * @param compareCol
     * @param isfromDb
     * @param entityCode
     * @param qFilters
     * @param dataContainer
     * @description: 全量构造器
     * @return:
     * @author rd_ningjie
     * @date: 2022/6/6 10:24
     */
    public CompareEntity(String relactionCol, String compareCol, String compareValue, boolean isfromDb, String entityCode, QFilter[] qFilters, Map<String, DynamicObject> dataContainer) {
        this.relactionCol = relactionCol;
        this.compareCol = compareCol;
        this.compareValue = compareValue;
        this.isfromDb = isfromDb;
        this.entityCode = entityCode;
        this.qFilters = qFilters;
        this.dataContainer = dataContainer;
    }

    /**
     * @param relactionCol
     * @param compareCol
     * @param dataContainer
     * @description: 非数据库数据源获取对象方法
     * @return: kd.bos.util.cascadeDataValide.CompareEntity
     * @author rd_ningjie
     * @date: 2022/6/6 11:20
     */
    public static CompareEntity newInstance(String relactionCol, String compareCol, String compareValue, Map<String, DynamicObject> dataContainer) {
        return new CompareEntity(relactionCol, compareCol, compareValue, false, null, null, dataContainer);
    }

    /**
     * @param relactionCol
     * @param compareCol
     * @param entityCode
     * @param qFilters
     * @description: 数据库数据源获取对象方法
     * @return: kd.bos.util.cascadeDataValide.CompareEntity
     * @author rd_ningjie
     * @date: 2022/6/6 11:20
     */
    public static CompareEntity newInstanceFromDb(String relactionCol, String compareCol, String compareValue, String entityCode, QFilter[] qFilters) {
        return new CompareEntity(relactionCol, compareCol, compareValue, true, entityCode, qFilters, null);
    }


    /**
     * @description: 从数据库查询并构造数据源
     * @return: void
     * @author rd_ningjie
     * @date: 2022/6/6 10:46
     */
    public void initData() {
        dataContainer = ObjectUtils.isEmpty(dataContainer) ? new HashMap<>() : dataContainer;
        //如果非数据库数据源，则直接return
        if (!this.isfromDb) {
            return;
        }
        //查询关联字段、对比字段、以及自定义查询字段
        String selectProps = this.relactionCol + "," + this.compareCol;
        if (HRStringUtils.isNotEmpty(this.selectExtProps)) {
            selectProps += "," + this.selectExtProps;
        }

        String finalSelectProps = selectProps;
        Optional.ofNullable(this.entityCode).ifPresent(code -> {
            HRBaseServiceHelper dataHelper = new HRBaseServiceHelper(code);
            DynamicObject[] results = dataHelper.query(finalSelectProps, this.qFilters);
            Optional.ofNullable(results).ifPresent(datas -> {
                //先初始化数据源
                dataContainer = new HashMap();
                Arrays.stream(datas).forEach(data -> {
                    //关联字段与对比字段联合唯一，防止表中有单个字段重复的数据
                    dataContainer.put(data.getString(this.relactionCol) + CON_SYMBOL + data.getString(this.compareCol), data);
                });
            });
        });
    }


    public String getRelactionCol() {
        return relactionCol;
    }

    public String getCompareCol() {
        return compareCol;
    }

    public boolean isIsfromDb() {
        return isfromDb;
    }

    public String getEntityCode() {
        return entityCode;
    }

    public QFilter[] getQFilters() {
        return qFilters;
    }

    public void setEntityCode(String entityCode) {
        this.entityCode = entityCode;
    }

    public void setQFilters(QFilter[] qFilters) {
        this.qFilters = qFilters;
    }

    public void addQFilters(QFilter[] qFilters) {
        this.qFilters = (QFilter[]) ArrayUtils.add(qFilters, QFilter.class);
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public String getCompareValue() {
        return compareValue;
    }

    public void setCompareValue(String compareValue) {
        this.compareValue = compareValue;
    }

    public Map<String, DynamicObject> getDataContainer() {
        return dataContainer;
    }

    public void setDataContainer(Map<String, DynamicObject> dataContainer) {
        this.dataContainer = dataContainer;
    }

    public void setSelectExtProps(String selectExtProps) {
        this.selectExtProps = selectExtProps;
    }

}
