package kd.hrmp.hbjm.business.domain.application.impl.job;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.lang.Lang;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.constants.newhismodel.HisFieldNameConstants;
import kd.hr.hbp.common.util.HRStringUtils;
import kd.hrmp.hbjm.business.domain.application.job.IJobLevelAndGradeApplication;
import kd.hrmp.hbjm.common.constants.HBJMConstants;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

/**
 * @author rd_ningjie
 * @description:TODO
 * @date 2022/10/9 16:43
 */
public class JobLevelAndGradeApplicationImpl implements IJobLevelAndGradeApplication {
    /**
     * @param scmIds
     * @param names
     * @param lang
     * @description: 根据职级方案id与职级名称查询该语言环境下的数据
     * @return: kd.bos.dataentity.entity.DynamicObject[]
     * @author rd_ningjie
     * @date: 2022/10/9 16:42
     */
    @Override
    public DynamicObjectCollection getJobLevelByScmIdAndLangName(List<Long> scmIds, List<String> names, Lang lang) {
        ORM orm = ORM.create();
        orm.hint().setLang(lang);
        QFilter qfilter = new QFilter(HBJMConstants.JOBLEVELSCM, QCP.in, scmIds);
        qfilter.and(new QFilter(HRBaseConstants.NAME, QCP.in, names));
        qfilter.and(new QFilter(HBJMConstants.JOBLEVELSCM + "." +HBJMConstants.INIT_STATUS, QFilter.not_in,
                new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED}));
        QFilter enableFilter = new QFilter(HBJMConstants.JOBLEVELSCM + "." + HRBaseConstants.ENABLE,
                QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        qfilter.and(enableFilter);
        qfilter.and(new QFilter(HBJMConstants.JOBLEVELSCM + "." +HBJMConstants.INIT_STATUS, QFilter.not_in,
                new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED}));

        qfilter.and(new QFilter(HBJMConstants.JOBLEVELSCM + "." + HisFieldNameConstants.IS_CURRENT_VERSION, QCP.equals, HRBaseConstants.STR_ONE));
        DynamicObjectCollection ormResult = orm.query(HBJMConstants.PAGE_JOBLEVEL, "id,number,name,joblevelseq,enable,joblevelscm.id,joblevelscm.name", new QFilter[]{qfilter});
        return ormResult;
    }

    /**
     * @param scmIds
     * @param names
     * @param lang
     * @description: 根据职等方案id与职等名称查询该语言环境下的数据
     * @return: kd.bos.dataentity.entity.DynamicObject[]
     * @author rd_ningjie
     * @date: 2022/10/9 16:42
     */
    @Override
    public DynamicObjectCollection getJobGradeByScmIdAndLangName(List<Long> scmIds, List<String> names, Lang lang) {
        ORM orm = ORM.create();
        orm.hint().setLang(lang);
        QFilter qfilter = new QFilter(HBJMConstants.JOBGRADESCM, QCP.in, scmIds);
        qfilter.and(new QFilter(HRBaseConstants.NAME, QCP.in, names));
        qfilter.and(new QFilter(HBJMConstants.JOBGRADESCM + "." +HBJMConstants.INIT_STATUS, QFilter.not_in,
                new String[]{HBJMConstants.INIT_STATUS_ING, HBJMConstants.INIT_STATUS_VALIDATED}));
        QFilter enableFilter = new QFilter(HBJMConstants.JOBGRADESCM + "." + HRBaseConstants.ENABLE,
                QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        qfilter.and(enableFilter);
        qfilter.and(new QFilter(HBJMConstants.JOBGRADESCM + "." + HisFieldNameConstants.IS_CURRENT_VERSION, QCP.equals, HRBaseConstants.STR_ONE));
        DynamicObjectCollection ormResult = orm.query(HBJMConstants.PAGE_JOBGRADE, "id,number,name,jobgradeseq,enable,jobgradescm.id,jobgradescm.name", new QFilter[]{qfilter});
        return ormResult;
    }


    @Override
    public DynamicObject[] getScmByNumberOrName(Set<String> scmNumber, Set<String> scmName, String entityName) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(entityName);
        QFilter qfilter = new QFilter(HisFieldNameConstants.IS_CURRENT_VERSION, QCP.equals, Boolean.TRUE);
        QFilter nfilter = new QFilter(HRBaseConstants.NUMBER, QCP.in, scmNumber).or(new QFilter(HRBaseConstants.NAME, QCP.in, scmName));
        qfilter.and(nfilter);

        String selectFields = "";
        if(HRStringUtils.equals(entityName, HBJMConstants.PAGE_JOBGRADESCM)){
            selectFields = "id, boid, name, number, entryentity, entryentity.jobgrade_number, entryentity.jobgrade_name ";
        }else {
            selectFields = "id, boid, name, number, entryentity, entryentity.joblevel_number, entryentity.joblevel_name ";
        }

        return serviceHelper.query(selectFields, new QFilter[]{qfilter});
    }
}