package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.HBJMLeftTreeConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.Arrays;
import java.util.List;

/**
 * @author rd_yan_h
 * @desc 职位分类方案类数据库操作Repository
 * @create 2021/12/06 10:14
 **/
public class JobClassScmRepository {
    private static class JobClassScmInstance {
        private static JobClassScmRepository INSTANCE = new JobClassScmRepository();
    }

    public static JobClassScmRepository getInstance() {
        return JobClassScmInstance.INSTANCE;
    }

    public DynamicObject[] queryJobClassScm(Long jobScmId, Long jobSeqId, Long jobFamilyId, Boolean isShowDisable) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter("jobclass.enable", QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        } else {
            enableFilter = new QFilter("jobclass.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        QFilter scmFilter = new QFilter(HBJMLeftTreeConstants.JOB_SCM, QFilter.equals, jobScmId);
        QFilter seqFilter = new QFilter("jobfamily.jobseq", QFilter.equals, jobSeqId);
        QFilter familyFilter = new QFilter(HBJMLeftTreeConstants.JOB_FAMILY, QFilter.equals, jobFamilyId);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{scmFilter, enableFilter, seqFilter, familyFilter});
    }

    public DynamicObject[] queryJobClassScm(List<Long> jobScmId, List<Long> jobSeqId, List<Long> jobFamilyId, Boolean isShowDisable, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter("jobclass.enable", QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        } else {
            enableFilter = new QFilter("jobclass.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        QFilter scmFilter = new QFilter(HBJMLeftTreeConstants.JOB_SCM, QFilter.in, jobScmId);
        QFilter seqFilter = new QFilter("jobfamily.jobseq", QFilter.in, jobSeqId);
        QFilter familyFilter = new QFilter(HBJMLeftTreeConstants.JOB_FAMILY, QFilter.in, jobFamilyId);
        QFilter initstatusFilter = new QFilter("jobclass.initstatus", QFilter.not_in, new String[]{"0", "1"});
        return serviceHelper.queryOriginalArray("jobclass.name,jobscm,jobfamily.jobseq,jobfamily,jobclass,jobclass.enable,jobclass.parent", new QFilter[]{scmFilter, enableFilter, seqFilter, familyFilter, initstatusFilter}, orderCondition);
    }

    /**
     * 根据职位体系方案id查询下面的职位类
     *
     * @param jobScmId 职位体系方案id
     * @return dyns 职位体系方案下的职位类集合
     */
    public DynamicObject[] queryJobClassScmByScmId(Long jobScmId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, jobScmId);
        return serviceHelper.query("id,name,number,jobfamily,jobclass,createorg,highjobgrade,lowjobgrade,highjoblevel,lowjoblevel", new QFilter[]{scmFilter, enableFilter}, "number asc,id desc");
    }

    /**
     * 根据职位体系方案id查询下面的职位类
     *
     * @param jobScmId 职位体系方案id
     * @param classIds 职位族id集合
     * @return dyns 职位体系方案下的职位类集合
     */
    public DynamicObject[] queryJobClassScm(Long jobScmId, List<Long> classIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, jobScmId);
        QFilter classFilter = new QFilter("jobclass", QFilter.in, classIds);
        return serviceHelper.query("id,name,number,jobfamily,jobclass,createorg,highjobgrade,lowjobgrade,highjoblevel,lowjoblevel", new QFilter[]{scmFilter, enableFilter, classFilter}, "number asc,id desc");
    }

    /**
     * 批量保存职位序列
     *
     * @param jobClassCollection 职位序列集合
     */
    public void saveBatch(DynamicObjectCollection jobClassCollection) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        serviceHelper.save(jobClassCollection);
    }

    /**
     * 删除职位体系方案关联的职位族
     *
     * @param scmId 职位体系方案Id
     */
    public void deleteByScmId(Long scmId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, scmId);
        serviceHelper.deleteByFilter(new QFilter[]{scmFilter});
    }

    /**
     * 删除职位体系方案关联的职位族
     *
     * @param scmIds 职位体系方案Id
     */
    public void deleteByScmIds(List<Long> scmIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.in, scmIds);
        serviceHelper.deleteByFilter(new QFilter[]{scmFilter});
    }

    /**
     * 根据职位类id查询最高最低职级职等
     * @param jobScmIds jobScmIds
     * @param jobClassIds jobClassIds
     * @return dys
     */
    public DynamicObject[] queryJobClassLevelGradeInfo(List<Long> jobScmIds,List<Long> jobClassIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        return serviceHelper.query("jobscm,jobclass,lowjoblevel,highjoblevel,lowjobgrade,highjobgrade",
                new QFilter[]{new QFilter(JobConstans.JOBCLASS_ID, QFilter.in, jobClassIds),
                        new QFilter(JobConstans.JOBSCM_ID, QFilter.in, jobScmIds)});
    }

    /**
     * 根据职位体系方案ID与职位类ID
     *
     * @param jobScmIds   职位体系ID(批量)
     * @param jobClassIds 职位类ID(批量)
     * @return 获取职位体系方案类的信息, 后续自行处理
     */
    public DynamicObject[] getJobClassScmDyobjsByJobScmIdsAndJobClassIds(List<Long> jobScmIds,
                                                                         List<Long> jobClassIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter qFilter = new QFilter("jobscm.id", QFilter.in, jobScmIds);
        qFilter.and(new QFilter("jobclass.id", QFilter.in, jobClassIds));

        return serviceHelper.query("id,name,number,jobscm,jobfamily,jobclass,createorg,highjobgrade,lowjobgrade,highjoblevel,lowjoblevel", new QFilter[]{qFilter});
    }

    /**
     * 根据职位体系方案编码查询职位体系方案下类
     * @param scmnumber
     * @return dy
     */
    public DynamicObject getJobClassScmdyn(String scmnumber) {
        HRBaseServiceHelper classcmserviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        return classcmserviceHelper.queryOne("jobclass,jobclass.number,jobclass.name,jobfamily", new QFilter("jobscm.number", QCP.equals, scmnumber));
    }

    /**
     * 根据职位体系方案id和职位族查询下面的类
     * @param jobScmIds 职位体系方案id集合
     * @param classIds 职位类id集合
     * @return dyns 职位体系方案下的职位类集合
     */
    public DynamicObject[] queryJobClassScmByScmIdsAndFamilyIds(List<Long> jobScmIds, List<Long> classIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.in, jobScmIds);
        QFilter jobclassFilter = new QFilter("jobclass", QFilter.in, classIds);
        return serviceHelper.loadDynamicObjectArray(new QFilter[]{scmFilter,jobclassFilter});
    }

}
