package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.HBJMLeftTreeConstants;
import kd.hrmp.hbjm.common.constants.JobConstans;

import java.util.Arrays;
import java.util.List;

/**
 * @author rimwood
 * @desc 职位体系方案下序列数据库操作Repository
 * @create 2021-12-10 09:37:42
 **/
public class JobSeqScmRepository {
    /**
     * 日志
     */
    private static final Log logger = LogFactory.getLog(JobSeqScmRepository.class);

    private static class JobSeqScmInstance {
        private static JobSeqScmRepository INSTANCE = new JobSeqScmRepository();
    }

    public static JobSeqScmRepository getInstance() {
        return JobSeqScmInstance.INSTANCE;
    }

    /**
     * 根据职位体系方案id查询下面的序列
     *
     * @param jobScmId 职位体系方案id
     * @return dyns 职位体系方案下的职位序列集合
     */
    public DynamicObject[] queryJobSeqScmByScmId(Long jobScmId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBSEQSCM);
        QFilter enableFilter = new QFilter(HRBaseConstants.ENABLE, QFilter.equals, HRBaseConstants.ENABLED);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, jobScmId);
        return serviceHelper.query("id,name,number,jobseq,createorg", new QFilter[]{scmFilter, enableFilter}, "number asc,id desc");
    }

    public DynamicObject[] queryJObSeqScmByScmId(List<Long> jobScmId, Boolean isShowDisable, String orderCondition) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBSEQSCM);
        QFilter enableFilter;
        if (isShowDisable) {
            enableFilter = new QFilter("jobseq.enable", QFilter.in, Arrays.asList(new String[]{HRBaseConstants.ENABLED, HRBaseConstants.DISABLED}));
        } else {
            enableFilter = new QFilter("jobseq.enable", QFilter.equals, HRBaseConstants.ENABLED);
        }
        QFilter scmFilter = new QFilter(HBJMLeftTreeConstants.JOB_SCM, QFilter.in, jobScmId);
        QFilter initstatusFilter=new QFilter("jobseq.initstatus",QFilter.not_in,new String[]{"0","1"});
        return serviceHelper.queryOriginalArray("jobseq.name,jobseq,jobseq.enable,jobscm", new QFilter[]{scmFilter, enableFilter,initstatusFilter}, orderCondition);
    }

    /**
     * 批量保存职位序列
     *
     * @param jobSeqCollection 职位序列集合
     */
    public void saveBatch(DynamicObjectCollection jobSeqCollection) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBSEQSCM);
        serviceHelper.save(jobSeqCollection);
    }

    /**
     * 删除职位体系方案关联的职位序列
     *
     * @param scmId 职位体系方案Id
     */
    public void deleteByScmId(Long scmId) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBSEQSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.equals, scmId);
        serviceHelper.deleteByFilter(new QFilter[]{scmFilter});
    }

    /**
     * 删除职位体系方案关联的职位序列
     *
     * @param scmIds 职位体系方案Id
     */
    public void deleteByScmIds(List<Long> scmIds) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBSEQSCM);
        QFilter scmFilter = new QFilter("jobscm", QFilter.in, scmIds);
        serviceHelper.deleteByFilter(new QFilter[]{scmFilter});
    }

    /**
     * 根据方案id查询职位体系方案 序列
     *
     * @param id
     * @return
     */
    public static DynamicObject[] getJobSeqByScmId(Long id) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBSEQSCM);
        QFilter idFilter = new QFilter(JobConstans.JOBSCM, QFilter.in, id);
        QFilter[] idFilterArray = new QFilter[]{idFilter};
        DynamicObject[] infoObject = serviceHelper.queryOriginalArray(
                JobConstans.JOBSEQ_ID,
                idFilterArray);

        return infoObject;
    }

    /**
     * 根据方案id查询职位体系方案族
     *
     * @param id
     * @return
     */
    public static DynamicObject[] getJobFamilyByScmId(Long id) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBFAMILYSCM);
        QFilter idFilter = new QFilter(JobConstans.JOBSCM, QFilter.in, id);
        QFilter[] idFilterArray = new QFilter[]{idFilter};
        DynamicObject[] infoObject = serviceHelper.queryOriginalArray(
                JobConstans.JOBFAMILY_ID,
                idFilterArray);

        return infoObject;
    }

    /**
     * 根据方案id查询职位体系方案类
     *
     * @param id
     * @return
     */
    public static DynamicObject[] getJobClassByScmId(Long id) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(HBJMConstants.PAGE_JOBCLASSSCM);
        QFilter idFilter = new QFilter(JobConstans.JOBSCM, QFilter.in, id);
        QFilter[] idFilterArray = new QFilter[]{idFilter};
        DynamicObject[] infoObject = serviceHelper.queryOriginalArray(
                JobConstans.JOBCLASS_ID,
                idFilterArray);

        return infoObject;
    }
}
