package kd.hrmp.hbjm.business.init;

import com.google.common.collect.Maps;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hrmp.hbjm.business.utils.GetDataFromMemory;
import kd.hrmp.hbjm.common.constants.HBJMConstants;
import kd.hrmp.hbjm.common.constants.JobClassTreeConstants;

import java.util.List;
import java.util.Map;

/**
 * @author rd_ningjie
 * @description: 职位类初始化消费类
 * @date 2022/6/21 11:38
 */
public class JobClassInitComsumerService extends HisBaseDataComsumerService {

    private static Log LOG = LogFactory.getLog(JobClassInitComsumerService.class);


    /**
     * @param dyns
     * @description: 扩展职位类数据处理功能，进行对上级职位类的数据处理
     * @return: void
     * @author rd_ningjie
     * @date: 2022/6/21 11:39
     */
    @Override
    protected void dealDynData(List<DynamicObject> dyns, List<Map<String, Object>> datamaps) {
        super.dealDynData(dyns, datamaps);
        LOG.info("JobClassInitComsumerService dealDynData start", dyns);
        //获取map源数据，初始化不做上级职位类的校验，需要获取数据源进行对象的构建
        if (!ObjectUtils.isEmpty(dyns) && !ObjectUtils.isEmpty(datamaps)) {
            Map<String, String> classIdMap = Maps.newHashMapWithExpectedSize(datamaps.size());
            //构建主id与上级职位类编码的映射
            for (Map<String, Object> datamap : datamaps) {
                classIdMap.put(String.valueOf(datamap.get(HRBaseConstants.NUMBER)), String.valueOf(datamap.get(JobClassTreeConstants.PARENT)));
            }
            GetDataFromMemory getDataFromMemory = new GetDataFromMemory();
            getDataFromMemory.initData(new String[]{HRBaseConstants.NUMBER}, dyns);
            for (DynamicObject dyna : dyns) {
                String number = dyna.getString(HRBaseConstants.NUMBER);
                //如果为空，则从内存中获取放入一次
                if(ObjectUtils.isEmpty(dyna.getDynamicObject(JobClassTreeConstants.PARENT))){
                    dyna.set(JobClassTreeConstants.PARENT, getDataFromMemory.getDynamicObjectByKey(classIdMap.get(number)));
                }

                //职位序列，如果职位序列为空，则可以从职位族上获取
                DynamicObject jobSeq = dyna.getDynamicObject(HBJMConstants.JOB_SEQ);
                if (ObjectUtils.isEmpty(jobSeq)) {
                    DynamicObject jobFamily = dyna.getDynamicObject(HBJMConstants.JOBFAMILY);
                    if (!ObjectUtils.isEmpty(jobFamily)) {
                        dyna.set(HBJMConstants.JOB_SEQ, jobFamily.get(HBJMConstants.JOB_SEQ));
                    }
                }

            }
        }
        LOG.info("JobClassInitComsumerService dealDynData end");
    }
}
